# @push.rocks/smartcli

easy observable cli tasks

## Install

To install `@push.rocks/smartcli`, use the following command in your terminal:

```sh
npm install @push.rocks/smartcli --save
```

This will add `@push.rocks/smartcli` as a dependency to your project's `package.json` file and download it into the `node_modules` folder.

## Usage

The `@push.rocks/smartcli` module is designed to simplify the creation of command-line interfaces (CLIs) by providing an easy-to-use API for managing CLI commands and options. It combines observables with the parsing power of [yargs-parser](https://www.npmjs.com/package/yargs-parser) to offer a dynamic and flexible way to handle various CLI tasks.

### Getting Started

First, ensure you have TypeScript and the necessary types for node installed in your project. If not, you can add them by running:

```sh
npm install typescript @types/node --save-dev
```

Then, import the `Smartcli` class from the `@push.rocks/smartcli` package.

```typescript
import { Smartcli } from '@push.rocks/smartcli';
```

### Creating an Instance

Create an instance of `Smartcli`. This instance will be used to define and manage your CLI commands.

```typescript
const mySmartcli = new Smartcli();
```

### Defining Commands

With `Smartcli`, you can define commands that your CLI tool can execute. Here's how you can add a new command:

```typescript
mySmartcli.addCommand('install').subscribe((argv) => {
  console.log('Install command was called with arguments:', argv);
});
```

In this example, when the user types `install` after your CLI tool's name in the terminal, the provided function will execute, printing the parsed arguments to the console.

### Handling Options

Options can be accessed using the `getOption` method. If you have an option named `--config` or `-c`, you can access its value like this:

```typescript
const configValue = mySmartcli.getOption('config');
console.log('Config value:', configValue);
```

### Default Task (Standard Command)

If you want to perform a task when no specific command is provided, you can use the `standardCommand` method:

```typescript
mySmartcli.standardCommand().subscribe((argv) => {
  console.log('No specific command provided. Running default task with arguments:', argv);
});
```

### Version and Help

To add a version option that prints the version of your CLI tool, use `addVersion`:

```typescript
mySmartcli.addVersion('1.0.0');
```

For a help option that displays helpful information about your CLI commands and options, use `addHelp`:

```typescript
mySmartcli.addHelp({
  helpText: 'Here are the commands you can use...'
});
```

### Parsing and Execution

After defining all your commands and options, call `startParse` to begin parsing the command line input and execute the corresponding actions:

```typescript
mySmartcli.startParse();
```

### Advanced Usage: Command Aliases

You can also define aliases for your commands, allowing users to use alternate names for them:

```typescript
mySmartcli.addCommandAlias('install', 'i');
```

With this setup, both `install` and `i` will trigger the same command action.

### Observables and Async Operations

Since commands in `smartcli` are handled using observables, you can easily integrate asynchronous operations within your command actions. This makes it perfect for CLI tasks that involve file operations, network requests, or any other async tasks.

### Conclusion

`@push.rocks/smartcli` offers a robust and intuitive way to build CLI tools with TypeScript, leveraging the reactive programming paradigm. By following the examples provided in this guide, you'll be able to create a feature-rich command-line application tailored to your specific needs.

Remember, this is just the start. The true power of `smartcli` lies in its flexibility and the vast ecosystem of RxJS. Dive into the RxJS documentation to explore more ways to handle data streams and asynchronous events in your CLI app.

## License and Legal Information

This repository contains open-source code that is licensed under the MIT License. A copy of the MIT License can be found in the [license](license) file within this repository. 

**Please note:** The MIT License does not grant permission to use the trade names, trademarks, service marks, or product names of the project, except as required for reasonable and customary use in describing the origin of the work and reproducing the content of the NOTICE file.

### Trademarks

This project is owned and maintained by Task Venture Capital GmbH. The names and logos associated with Task Venture Capital GmbH and any related products or services are trademarks of Task Venture Capital GmbH and are not included within the scope of the MIT license granted herein. Use of these trademarks must comply with Task Venture Capital GmbH's Trademark Guidelines, and any usage must be approved in writing by Task Venture Capital GmbH.

### Company Information

Task Venture Capital GmbH  
Registered at District court Bremen HRB 35230 HB, Germany

For any legal inquiries or if you require further information, please contact us via email at hello@task.vc.

By using this repository, you acknowledge that you have read this section, agree to comply with its terms, and understand that the licensing of the code does not imply endorsement by Task Venture Capital GmbH of any derivative works.
